# --
# Kernel/System/Auth/HTTPBasicAuth.pm - provides the $ENV authentication
# Copyright (C) 2001-2009 OTRS AG, http://otrs.org/
# --
# $Id: HTTPBasicAuth.pm,v 1.16 2009/09/22 15:19:27 mb Exp $
# --
# This software comes with ABSOLUTELY NO WARRANTY. For details, see
# the enclosed file COPYING for license information (AGPL). If you
# did not receive this file, see http://www.gnu.org/licenses/agpl.txt.
# --
# Note:
#
# If you use this module, you should use as fallback the following
# config settings:
#
# If use isn't login through apache ($ENV{REMOTE_USER} or $ENV{HTTP_REMOTE_USER})
# $Self->{LoginURL} = 'http://host.example.com/not-authorised-for-otrs.html';
#
# $Self->{LogoutURL} = 'http://host.example.com/thanks-for-using-otrs.html';
# --

package Kernel::System::Auth::OpMon;

use strict;
use warnings;
use DBI;
use Data::Dumper;
use CGI::Cookie;
use Net::LDAP;
use Kernel::System::Auth::LDAP;


use vars qw($VERSION);
$VERSION = qw($Revision: 0.1 $) [1];

sub new {
    my ( $Type, %Param ) = @_;

    # allocate new hash for object
    my $Self = {};
    bless( $Self, $Type );

    # check needed objects
    for (qw(LogObject ParamObject ConfigObject DBObject)) {
        $Self->{$_} = $Param{$_} || die "No $_!";
    }

	# OpMon Config File
	$Self->{dbConfigFile} = $Self->{ConfigObject}->Get( 'OpMon::DBConfigFile' );
	if (!($Self->{dbConfigFile})) {
		$Self->{LogObject}->Log(
			Priority => 'error',
			Message  => "Need OpMon::DBConfigFile in Kernel/Config.pm",
		);
		return;
    }

    # Debug 0=off 1=on
    $Self->{Debug} = 0;
    $Self->{Count} = $Param{Count} || '';

    return $Self;
}

sub GetOption {
    my ( $Self, %Param ) = @_;

    # check needed stuff
    if ( !$Param{What} ) {
        $Self->{LogObject}->Log( Priority => 'error', Message => "Need What!" );
        return;
    }

    # module options
    my %Option = ( PreAuth => 1, );

    # return option
    return $Option{ $Param{What} };
}

sub Auth {
    my ( $Self, %Param ) = @_;

	my $Request = $Self->{ParamObject}->GetParam( Param => 'RequestedURL' );
	my @pairs = split(/&/, $Request);
	foreach my $pair (@pairs) {
		my ($name, $value) = split(/=/,$pair);
		$Param{$name} = $value;
	}
	
    # get params
	my $User = $Param{User} || '';
	my $Pw = $Param{Pw} || '';
	my $SessionID = $Param{SessionID} || '';

    # return on no user
    if ( !$User ) {
        $Self->{LogObject}->Log(
            Priority => 'notice',
            Message =>
                "User: No User!.",
        );
        return;
    }

	# read cookies
	my %cookies = fetch CGI::Cookie;
	if (defined($cookies{'OPMONSSID'})) {
		my $MySessionID = $cookies{'OPMONSSID'}->value;
		if ($MySessionID eq $SessionID) {
			return $User;
		}
	}

	# parse config file
	my ($dbuser,$dbpass,$dbhost);
	open( FILE, $Self->{dbConfigFile} );
	while(<FILE>) {
		$dbuser = $1 if (m/DBUSER\s*=\s*[\'\"]*([^\'\"]*)[\'\"]*;/);
		$dbpass = $1 if (m/DBPASS\s*=\s*[\'\"]*([^\'\"]*)[\'\"]*;/);
		$dbhost = $1 if (m/DBHOST\s*=\s*[\'\"]*([^\'\"]*)[\'\"]*;/);
	}
	close(FILE);

	# validate password
	my $dbh = DBI->connect( "dbi:mysql:dbname=opcfg;host=$dbhost", "$dbuser", "$dbpass" );
	if (!defined($dbh)) {
		$Self->{LogObject}->Log( Priority => 'error', Message => "Can't connect to database!" );
		return;
	}

	
	# just escape user and pw for query
	$User =~ s/\'/\\'/g;
	$User =~ s/\"/\\"/g;	
	
	$Pw =~ s/\'/\\'/g;
	$Pw =~ s/\"/\\"/g;	
	
	# check if user uses external auth
	my $sth2 = $dbh->prepare("SELECT external_auth FROM nagios_contacts WHERE contact_name = '$User'");
	$sth2->execute() or die $_;
	my $authType = $sth2->fetchrow_array();
		
	if($authType == 0) {
		my $sth = $dbh->prepare("SELECT contact_id FROM nagios_contacts WHERE contact_name = '$User' AND password = md5('$Pw')");
		if (!defined($sth)) {
			$Self->{LogObject}->Log( Priority => 'error', Message => "Cannot prepare statement: $DBI::errstr!" );
			return;
		}

		$sth->execute;
		my @Data = $sth->fetchrow_array();
		if (!(@Data)) {
			$Self->{LogObject}->Log( Priority => 'error', Message => "Wrong password for user $User!" );
			return;
		}
		$sth->finish;
		
	} elsif($authType == 1) { 
		# get ldap info
		my $sth = $dbh->prepare("SELECT ext_auth_domain_controllers, ext_auth_base_dn, ext_auth_ad_username, ext_auth_ad_password, 
								 ext_auth_ous, ext_auth_suffix, ext_auth_type, ext_auth_attr FROM nagios_main LIMIT 1");
		$sth->execute() or die $_;
		
		my ($ext_auth_domain_controllers, $ext_auth_base_dn, $ext_auth_ad_username, $ext_auth_ad_password, $ext_auth_ous, $ext_auth_suffix, $ext_auth_type, $ext_auth_attr);
		$sth->bind_columns(\($ext_auth_domain_controllers, $ext_auth_base_dn, $ext_auth_ad_username, $ext_auth_ad_password, $ext_auth_ous, $ext_auth_suffix, $ext_auth_type, $ext_auth_attr));
		$sth->fetch;		
	
		my $LDAP = Net::LDAP->new($ext_auth_domain_controllers, version => 3, verify => 'none') or die "$@";
	
		my $dn = "dc=".join(',dc=', split(/\./, $ext_auth_base_dn));
		my $query = "";
		
		# check auth type
		# 'ad' and 'custom' are disabled since
		# only ldap is used
		if($ext_auth_type eq 'ad') { # ad
			$query = $ext_auth_attr."=".$User.",ou=".$ext_auth_ous.",".$dn;
		} elsif($ext_auth_type eq 'custom') { # custom	
			$query = $ext_auth_attr."=".$User.",ou=".$ext_auth_ous.",".$dn;
		} else { # ldap
			my $OUs = "ou=".join(',ou=',split(/\,/,$ext_auth_ous));
			$query = $ext_auth_attr."=".$User.",".$OUs.",".$dn;
		}
			
		if(!$LDAP) {
			$Self->{LogObject}->Log(
				Priority => 'error',
				Message  => 'Failed to connect: '. $ext_auth_base_dn,
			);
			return;
		}

		my $Result = '';			
		$Result = $LDAP->bind($query, password => $Pw);
						
		if($Result->code) {
			$Self->{LogObject}->Log(
				Priority => 'error',
				Message  => 'Search failed! ' . $Result->error,
			);			
			# take down session
			$LDAP->unbind;
			$LDAP->disconnect;
			return;
		}
			
		$LDAP->unbind;
		$LDAP->disconnect;
		
		$sth->finish;
	} else { 
		$Self->{LogObject}->Log( Priority => 'error', Message => "Wrong password for user $User!" );
		return;
	}
	$sth2->finish; 

    # return login
    return $User;
}
1;
